@@ -57,13 +57,6 @@ as that of the covered work.  */
 /* Declares getpid(). */
 #include <process.h>
 
-#ifndef S_ISDIR
-# define S_ISDIR(m) (((m) & (_S_IFMT)) == (_S_IFDIR))
-#endif
-#ifndef S_ISLNK
-# define S_ISLNK(a) 0
-#endif
-
 /* We have strcasecmp and strncasecmp, just under different names.  */
 #ifndef HAVE_STRCASECMP
 # define strcasecmp stricmp
@@ -74,10 +67,6 @@ as that of the covered work.  */
 
 #include <stdio.h>
 
-/* The same for snprintf() and vsnprintf().  */
-#define snprintf _snprintf
-#define vsnprintf _vsnprintf
-
 /* Define a wgint type under Windows. */
 typedef __int64 wgint;
 #define SIZEOF_WGINT 8
@@ -94,18 +83,6 @@ typedef __int64 wgint;
 # define strtoll_type __int64
 #endif
 
-/* Windows has no symlink, therefore no lstat.  Without symlinks lstat
-   is equivalent to stat anyway.  */
-#define lstat stat
-
-/* Define LFS aliases for stat and fstat. */
-#ifdef stat_alias
-# define stat(f, b) stat_alias (f, b)
-#endif
-#ifdef fstat_alias
-# define fstat(f, b) fstat_alias (f, b)
-#endif
-
 #define PATH_SEPARATOR '\\'
 
 /* Additional declarations needed for IPv6: */
